@@ -1,9 +1,12 @@
 import json
+import logging
+
 import boto3
+import fnmatch
 import os
 from datetime import datetime, timedelta
 from types import SimpleNamespace
-from typing import List, Optional, Tuple, Dict
+from typing import List, Optional, Tuple, Dict, Union
 
 from helix_catalog_sdk.data_source import DataSource, ResourceItem
 
@@ -17,8 +20,10 @@ class Catalog:
         repo: BaseRepo,
         environment: HelixEnvironment = HelixEnvironment.PRODUCTION,
     ):
-        self._repo = repo
-        self.environment = environment
+        self.logger = logging.getLogger(self.__class__.__name__)
+        self.logger.setLevel(os.environ.get("LOGLEVEL", "INFO"))
+        self._repo: BaseRepo = repo
+        self.environment: HelixEnvironment = environment
         self._last_updated_data_source_dttm: datetime = datetime.utcnow() + timedelta(
             days=-1
         )
@@ -28,7 +33,7 @@ class Catalog:
     def get_data_source(
         self, data_source: str, environment: HelixEnvironment
     ) -> Optional[DataSource]:
-        print(
+        self.logger.info(
             f"Data Catalog is reading data source: {data_source} for environment {environment}"
         )
         decoded_contents = self._repo.read_file(data_source)
@@ -36,7 +41,7 @@ class Catalog:
             decoded_contents, object_hook=lambda d: SimpleNamespace(**d)
         )
 
-        # Currently only data_sources of type "file" are working in the catalog.
+        # Currently, only data_sources of type "file" are working in the catalog.
         if getattr(contents, "connection_type", None) != "file":
             return None
 
@@ -76,21 +81,40 @@ class Catalog:
             r for r in data_source.resources if r.name == resource_name
         ][0]
         if resource.last_processed:
-            date_segment = resource.date_segment if resource.date_segment else 0
+            date_segment: Union[int, List[int]] = (
+                resource.date_segment if resource.date_segment else 0
+            )
+            self.logger.info(f"Using date_segment: {date_segment}")
 
-            date_format = resource.date_format if resource.date_format else "%Y-%m-%d"
-            last_processed = resource.last_processed.get_last_processed(environment)
-            all_paths = self.get_all_directories(resource, last_processed)
-            path_parts = list(
+            date_format: str = (
+                resource.date_format if resource.date_format else "%Y-%m-%d"
+            )
+            last_processed: str = resource.last_processed.get_last_processed(
+                environment
+            )
+            self.logger.info(
+                f"Found last processed path {last_processed} for resource {resource.name}"
+            )
+            all_paths: List[str] = self.get_all_directories(resource, last_processed)
+            self.logger.debug(
+                f"---- Found directories for path [{last_processed}] ------"
+            )
+            for path in all_paths:
+                self.logger.debug(f"{path}")
+            self.logger.debug(
+                "-----------------------------------------------------------"
+            )
+            path_parts: List[str] = list(
                 filter(None, last_processed.replace("s3:/", "").split("/"))
             )
 
+            last_processed_date: Optional[datetime] = None
             if isinstance(date_segment, int):
                 last_processed_date = datetime.strptime(
                     path_parts[date_segment], date_format
                 )
-            if isinstance(date_segment, list):
-                last_processed_date_str = ""
+            elif isinstance(date_segment, list):
+                last_processed_date_str: str = ""
                 for segment in date_segment:
                     last_processed_date_str = (
                         last_processed_date_str + path_parts[segment]
@@ -99,26 +123,51 @@ class Catalog:
                     last_processed_date_str, date_format
                 )
 
+            self.logger.info(
+                f"Using last_processed_date: {last_processed_date} {type(last_processed_date)}"
+            )
+
             for path in all_paths:
                 path_parts = list(filter(None, path.replace("s3:/", "").split("/")))
+                path_date: Optional[datetime] = None
                 if isinstance(date_segment, int):
                     path_date = datetime.strptime(path_parts[date_segment], date_format)
-                if isinstance(date_segment, list):
+                elif isinstance(date_segment, list):
                     path_date_str = ""
                     for segment in date_segment:
                         path_date_str = path_date_str + path_parts[segment]
                     path_date = datetime.strptime(path_date_str, date_format)
-                if path_date > last_processed_date:
+                if path_date is not None and (
+                    not last_processed_date or path_date > last_processed_date
+                ):
+                    self.logger.info(
+                        f"Adding path for {path_date} compared to {last_processed_date}"
+                    )
                     paths_to_process[path_date] = path
+                else:
+                    self.logger.debug(
+                        f"Skipped path for {path_date} compared to {last_processed_date}"
+                    )
 
             for path_date in sorted(paths_to_process):
                 sorted_paths_to_process.append(paths_to_process[path_date])
+
+            self.logger.info(
+                f"---- Sorted directories for path [{last_processed}] ------"
+            )
+            for path in sorted_paths_to_process:
+                self.logger.info(f"{path}")
+            self.logger.info(
+                "-----------------------------------------------------------"
+            )
+
             return sorted_paths_to_process
         else:
             return []
 
-    @staticmethod
-    def get_all_directories(resource: ResourceItem, last_processed: str) -> List[str]:
+    def get_all_directories(
+        self, resource: ResourceItem, last_processed: str
+    ) -> List[str]:
         full_path = resource.full_path
         all_paths: List[str] = []
         if full_path.startswith("s3://"):
@@ -126,14 +175,26 @@ class Catalog:
             bucket = path.split("/")[0]
             prefix = path.replace(f"{bucket}/", "")
             s3 = boto3.client("s3")
-            response = s3.list_objects_v2(Bucket=bucket, Prefix=prefix)
-            for s3_file in response["Contents"]:
-                if (
-                    len(list(filter(None, s3_file["Key"].split("/"))))
-                    == resource.date_segment
-                    or last_processed.split("/")[-1] == s3_file["Key"].split("/")[-1]
-                ):
-                    all_paths.append(f"s3://{bucket}/{s3_file['Key']}")
+            paginator = s3.get_paginator("list_objects_v2")
+            pages = paginator.paginate(Bucket=bucket, Prefix=prefix)
+            for response in pages:
+                for s3_file in response["Contents"]:
+                    s3_path_parts = s3_file["Key"].split("/")
+                    last_processed_file_name: str = last_processed.split("/")[-1]
+                    s3_file_name = s3_path_parts[-1]
+                    # 1. if they are exactly the same length then add this path
+                    # 2. if the file name matches then add this path
+                    # 3. if the glob pattern matches then add this path
+                    if (
+                        len(list(filter(None, s3_path_parts))) == resource.date_segment
+                        or last_processed_file_name == s3_file_name
+                        or fnmatch.fnmatch(s3_file_name, last_processed_file_name)
+                    ):
+                        all_paths.append(f"s3://{bucket}/{s3_file['Key']}")
+                    else:
+                        self.logger.debug(
+                            f"Did not match {s3_file['Key']} to {last_processed_file_name}"
+                        )
             return all_paths
         else:
             paths = [
@@ -155,7 +216,7 @@ class Catalog:
         )
         self._last_updated_data_source_dttm = datetime.utcnow()
         if needs_update or len(self._all_data_sources) == 0:
-            print("getting all data sources")
+            self.logger.info("Getting all data sources")
             catalog_contents = self._repo.list_items(base_path)
             data_sources: List[Tuple[str, DataSource]] = []
             while catalog_contents:
@@ -167,7 +228,8 @@ class Catalog:
                     if data_source:
                         data_sources.append((file_path, data_source))
             self._all_data_sources = data_sources
+            self.logger.info("Finished all data sources")
             return data_sources
         else:
-            print("returning cached data sources")
+            self.logger.info("Returning cached data sources")
             return self._all_data_sources
