@@ -35,9 +35,12 @@ class ResourceItem:
         self.last_processed = None
         if hasattr(content, "last_processed") and content.last_processed is not None:
             self.last_processed = LastProcessed(content.last_processed, environment)
+        self.path_extension = None
+        if hasattr(content, "path_extension") and content.path_extension is not None:
+            self.path_extension = content.path_extension
 
     @property
-    def full_path_slug(self) -> str:
+    def full_path_slug(self) -> Optional[str]:
         return self.path_slug and os.path.join(
             self.base_connection_formatted, self.path_slug
         )
@@ -48,7 +51,10 @@ class ResourceItem:
 
     def matches_path(self, path: str) -> bool:
         if self.full_path_slug and self.full_path_slug in path:
-            return True
+            if self.path_extension:
+                return path.endswith(self.path_extension)
+            else:
+                return True
         return False
 
     @property
@@ -141,7 +147,7 @@ class DataSource:
 
     def update_path_with_latest_file(self, file_path: str) -> None:
         resource = self.get_matching_resource(file_path)
-        if resource is not None:
+        if resource is not None and resource.path_slug is not None:
             index: int = file_path.find(resource.path_slug)
             resource.path = file_path[index:]
 
